//+------------------------------------------------------------------+
//|                                                         MACD.mq4 |
//|                                Copyright  2005, David W. Thomas |
//|                                           mailto:davidwt@usa.net |
//+------------------------------------------------------------------+
// This is the correct computation and display of MACD.
#property copyright "Copyright  2005, David W. Thomas"
#property link      "mailto:davidwt@usa.net"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  PaleGreen
#property indicator_color2  Blue
#property indicator_color3  Red
#property indicator_style3  STYLE_DOT

//
//
//
//
//

extern string TimeFrame         = "Current time frame";
extern int    FastMAPeriod      = 12;
extern int    SlowMAPeriod      = 26;
extern int    SignalMAPeriod    = 9;

extern string note              = "turn on Alert = true; turn off = false";
extern bool   alertsOn          = true;
extern bool   alertsOnCurrent   = true;
extern bool   alertsMessage     = true;
extern bool   alertsSound       = true;
extern bool   alertsEmail       = false;

//
//
//
//
//

double Macd[];
double Signal[];
double Histo[];
double trend[];

//
//
//
//
//

int    timeFrame;
string indicatorFileName;
bool   returnBars;
bool   calculateValue;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int init()
{  
   IndicatorBuffers(4);
   SetIndexBuffer(0,Histo); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,Macd);
   SetIndexBuffer(2,Signal);  
   SetIndexBuffer(3,trend);
   
     //
     //
     //
     //
     //
      
     indicatorFileName = WindowExpertName();
     calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
     returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
     timeFrame         = stringToTimeFrame(TimeFrame);
      
     //
     //
     //
     //
     //
   
   IndicatorShortName(timeFrameToString(timeFrame)+"  MACD("+FastMAPeriod+","+SlowMAPeriod+","+SignalMAPeriod+")");
 return(0);
}

//
//
//
//
//

int deinit(){  return(0); }

//
//
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start() {
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars)  { Histo[0] = limit+1; return(0); }
   
   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      double alpha = 2.0 / (SignalMAPeriod + 1.0);
      double alpha_1 = 1.0 - alpha;  
      for(i=limit; i>=0; i--)
      {
         Macd[i]  = iMA(NULL,0,FastMAPeriod,0,MODE_EMA,PRICE_CLOSE,i) - iMA(NULL,0,SlowMAPeriod,0,MODE_EMA,PRICE_CLOSE,i);
         Signal[i]= alpha * Macd[i] + alpha_1 * Signal[i+1];
         Histo[i] = Macd[i] - Signal[i];
         
         trend[i] = trend[i+1];
            if (Macd[i] > Signal[i]) trend[i] = 1;
            if (Macd[i] < Signal[i]) trend[i] =-1;
            
      }
      manageAlerts();
      return(0);            
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   alpha = 2.0 / (SignalMAPeriod + 1.0);
   alpha_1 = 1.0 - alpha;  
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         Macd[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastMAPeriod,SlowMAPeriod,SignalMAPeriod,1,y);
         Signal[i]= alpha * Macd[i] + alpha_1 * Signal[i+1];
         Histo[i] = Macd[i] - Signal[i];
         trend[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastMAPeriod,SlowMAPeriod,SignalMAPeriod,3,y); 
         
         }
   manageAlerts();
   return(0);
   }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"buy");
         if (trend[whichBar] ==-1) doAlert(whichBar,"sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" Macd ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Macd "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//
//
//
//
//
   